export type FileIntegrity = {
    algorithm: 'SHA256';
    hash: string;
    blockSize: number;
    blocks: string[];
};
export declare function getFileIntegrity(inputFileStream: NodeJS.ReadableStream): Promise<FileIntegrity>;
export declare function getFileIntegrityFromBuffer(data: Buffer): FileIntegrity;
