export declare const EXPANSION_MAX = 100000;
export type BraceExpansionOptions = {
    max?: number;
};
export declare function expand(str: string, options?: BraceExpansionOptions): string[];
//# sourceMappingURL=index.d.ts.map